import { Component, OnInit, Input, Output, EventEmitter, OnChanges, Inject, ViewChild, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { FormControl } from '@angular/forms'
import { Observable } from 'rxjs';
import { map, startWith, debounceTime, take } from 'rxjs/operators';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialog } from '@angular/material/dialog';
import { Globals } from 'src/app/core/global';


/**
 * How to use AutoCompleteQuerySearch
 * 1.create new tag name
 *    <auto-complete-query-search></auto-complete-query-search>
 * 2.add properties
 *    - config [Required][Input]
 *    - textInput [Not-required][Input]
 *    - disabled [Not-required][Input]
 *    - tabindex [Not-required][Input]
 *    - datacallback [Not-required][Output]
 * 
 * ***config property's model
 *  1. ColName: Key of object to show in screen.
 *  2. ColValue: Value for returning to callback (if this is not defined, return value will be a selected object.)
 *  3. PlaceholderText: A name that user understand what to do with this auto-complete
 *  4. SourceController
 *  5. SourceMethodName
 *  6. NoOfShowRecords
 *  7. SourceParams {Object}
 *  8. UpperCase
 */


@Component({
  selector: 'auto-complete-query-search',
  template: '<button type="button" mat-icon-button (click)="OpenDialog()" [disabled]="disabled" tabindex="{{tabindex}}" #sAutoCompleteBtn><i class="material-icons">arrow_drop_down</i></button>',
  styleUrls: ['./../autoComplete/auto-complete-btn.component.scss']
})

export class AutoCompleteQuerySearchComponent {

  constructor(
    public dialog: MatDialog,
    private http: HttpService,
    public global: Globals
  ) { }

  @Input() config: any;
  
  private _textInput: string;
  @Input()
  get textInput(): string {return this._textInput};
  set textInput(textInput: string) {
    this._textInput = textInput?.toString().trim() || "";
  }

  @Output() datacallback = new EventEmitter<string | object>();
  @Output() cancelcallback = new EventEmitter<string>();
  @Input() disabled: boolean = false;
  @Input() tabindex: number = -1;
  @ViewChild("sAutoCompleteBtn", { read: ElementRef }) sAutoCompleteBtn: ElementRef;
  @Input() width: string = '40%';

  private dialogRef: any;

  ngAfterViewInit() {
    if (this.config.UpperCase) {
      let inputElem = this.sAutoCompleteBtn.nativeElement.parentNode.parentNode.previousSibling.firstElementChild;

      if (inputElem && inputElem.nodeName === "INPUT") {
        inputElem.classList.add("text-uppercase");
      }
    }
  }

  ngOnDestroy() {
    setTimeout(() => {
      this.dialogRef ? this.dialogRef.close() : null;
    }, 100)
  }

  OpenDialog() {
    if (!this.disabled && (!this.dialogRef || this.dialogRef?.getState() === 2)) {
      this.dialogRef = this.dialog.open(AutoCompleteQuerySearchDialog, {
        width: this.width,
        data: {
          config: this.config,
          textInput: this.textInput,
          datacallback: this.datacallback,
          cancelcallback: this.cancelcallback
        }
      })

      this.dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe(result => { })
    }
  }

  query() {
    if (this.textInput && !this.disabled) {
      if (this.config.SourceParams && this.config.SourceParams !== {}) {
        postItemList(this.http, this.config, this.textInput, this.global)
          .subscribe(result => {
            let matchedValue: any;
            let matchedResult: any;

            if (this.config.ColValue) {
              //Find exactly match
              matchedValue = result.Table.filter(v => v[this.config.ColValue] === (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput));

              //Case not matched
              if (matchedValue.length === 0) {
                matchedValue = result.Table.filter(v =>
                  v[this.config.ColValue].indexOf((this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput)) >= 0
                );
              }
            } else {
              //Find exactly match
              matchedValue = result.Table.filter(v => {
                let mIndex = Object.keys(v).findIndex(k => {
                  return v[k] == (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput);
                })
                return mIndex !== -1;
              });

              //Case not matched
              if (matchedValue.length === 0) {
                matchedValue = result.Table.filter(v => {
                  let mIndex = Object.keys(v).findIndex(k => {
                    return v[k]?.toString().indexOf((this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput)) >= 0;
                  })
                  return mIndex !== -1;
                })
              }
            }

            if (matchedValue.length > 1) {
              this.OpenDialog();
            } else if (matchedValue.length === 1) {
              matchedResult = (this.config.ColValue) ? matchedValue[0][this.config.ColValue] : matchedValue[0];
              this.datacallback.emit(matchedResult);
            } else {
              this.OpenDialog();
              matchedResult = (this.config.ColValue) ? null : {};
            }

          })
      } else {
        getItemList(this.http, this.config, this.textInput, this.global)
          .subscribe(result => {
            let matchedValue: any;
            let matchedResult: any;

            if (this.config.ColValue) {
              //Find exactly match
              matchedValue = result.Table.filter(v => v[this.config.ColValue] === (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput));

              //Case not matched
              if (matchedValue.length === 0) {
                matchedValue = result.Table.filter(v =>
                  v[this.config.ColValue].indexOf((this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput)) >= 0
                );
              }
            } else {
              //Find exactly match
              matchedValue = result.Table.filter(v => {
                let mIndex = Object.keys(v).findIndex(k => {
                  return v[k] == (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput);
                })
                return mIndex !== -1;
              });

              //Case not matched
              if (matchedValue.length === 0) {
                matchedValue = result.Table.filter(v => {
                  let mIndex = Object.keys(v).findIndex(k => {
                    return v[k]?.toString().indexOf((this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput)) >= 0;
                  })
                  return mIndex !== -1;
                })
              }
            }

            if (matchedValue.length > 1) {
              this.OpenDialog();
            } else if (matchedValue.length === 1) {
              matchedResult = (this.config.ColValue) ? matchedValue[0][this.config.ColValue] : matchedValue[0];
              this.datacallback.emit(matchedResult);
            } else {
              this.OpenDialog();
              matchedResult = (this.config.ColValue) ? null : {};
            }
          })
      }
    } else if (!this.textInput) {
      this.datacallback.emit((this.config.ColValue) ? null : {})
    }
  }

}

@Component({
  selector: 'auto-complete-query-search-dialog',
  templateUrl: './auto-complete-query-search.component.html',
  styleUrls: ['./../autoComplete/auto-complete.component.scss', './auto-complete-query-search.component.css']
})
export class AutoCompleteQuerySearchDialog {

  public setting: any;
  private textInput: string = "";
  public selectedValue: any;
  public defaultMinLengthSearch: number = 3;
  public showSpinner: boolean = false;
  @ViewChild("selectBtn", { static: false }) selectBtn: any;

  myControl = new FormControl();
  valueControl = new FormControl();
  filteredOptions: Array<any> = [];

  constructor(
    public dialogRef: MatDialogRef<AutoCompleteQuerySearchDialog>,
    @Inject(MAT_DIALOG_DATA) public dialogData: any,
    private http: HttpService,
    public global: Globals
  ) { }

  ngOnInit() {
    this.setting = this.dialogData.config;
    this.textInput = this.dialogData.textInput?.toString() || "";
    this.myControl.setValue(this.textInput);

    this.valueControl.valueChanges
      .pipe(
        startWith(this.textInput),
        debounceTime(500),
        map(value => {
          this.filteredOptions = [];
          if (this.setting.MinLengthSearch >= 0) {
            return value.length >= this.setting.MinLengthSearch && (value || 'null')
          } else {
            return value.length >= this.defaultMinLengthSearch && value
          }
        })
      ).subscribe(value => {
        if (value) {
          this.GetDataByValue(value);
        }
      });

    if (this.textInput && this.textInput.length >= 0) {
      setTimeout(() => {
        this.GetDataByValue(this.textInput);
      }, 500);
    }

  }

  GetDataByValue(value) {
    this.showSpinner = true;
    this.filteredOptions = [];

    if (this.setting.SourceParams && this.setting.SourceParams !== {}) {
      postItemList(this.http, this.setting, value, this.global)
        .subscribe(result => {
          this.filteredOptions = result.Table;

          if (this.filteredOptions.length === 0) {
            this.filteredOptions.push({ [this.setting.ColName]: "No record found.", DISABLED: true });
          }
          this.showSpinner = false;
        }, (error) => {
          this.showSpinner = false;
        })
    } else {
      getItemList(this.http, this.setting, value, this.global)
        .subscribe(result => {
          this.filteredOptions = result.Table;

          if (this.filteredOptions.length === 0) {
            this.filteredOptions.push({ [this.setting.ColName]: "No record found.", DISABLED: true });
          }
          this.showSpinner = false;
        }, (error) => {
          this.showSpinner = false;
        })
    }
  }

  ClickCallback() {
    this.dialogData.datacallback.emit(this.selectedValue);
    this.dialogRef.close();
  }

  OnCancelClick() {
    if(this.dialogData.cancelcallback) {
      this.dialogData.cancelcallback.emit();
    }
    this.dialogRef.close();
  }

  SetValueControl(evt: any) {
    this.valueControl.setValue(this.myControl.value?.toString().trim())
  }

  SetValue(evt: any) {
    this.myControl.setValue(evt.option.value[this.setting.ColName]);
    this.selectedValue = (this.setting.ColValue) ? evt.option.value[this.setting.ColValue] : evt.option.value;

    this.selectBtn.focus();
  }

  SetClearValue() {
    this.selectedValue = (this.setting.ColValue) ? null : {};
    this.valueControl.setValue('')
    this.myControl.setValue('')
  }
}

function postItemList(http: HttpService, setting: any, value: string, global: Globals) {
  return http.autoCompletePost(
    setting.SourceController,
    [setting.SourceMethodName, global.EscapeCharacter((setting.UpperCase) ? value.toUpperCase() : value) || 'null', setting.NoOfShowRecords || 20],
    setting.SourceParams
  );
}

function getItemList(http: HttpService, setting: any, value: string, global: Globals) {
  return http.autoCompleteGet(setting.SourceController, setting.SourceMethodName, global.EscapeCharacter((setting.UpperCase) ? value.toUpperCase() : value) || 'null', setting.NoOfShowRecords || 20);
}