import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import * as moment from "moment";
import { Subject } from "rxjs";
import { debounceTime, distinctUntilChanged } from "rxjs/operators";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { VendorSelectComponent } from "./vendorSelect.component";

@Component({
  selector: "SNIC001",
  templateUrl: "SNIC001.component.html",
  styleUrls: ["SNIC001.component.css"],
})
export class SNIC001Component implements OnInit {
  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private dialog: MatDialog
  ) { }

  @ViewChild("tinymceEditorRef") tinymceEditorRef;
  public tinymceOptions: any = {};
  public vendorList: any[];
  public emailConfigureHeaders: any[];
  public selectedTabIndex: number = 0;
  public expandedGroupID: number = 0;

  ngOnInit() {
    this.GetEmailConfigure();
    this.GetVendorList();
    this.InitTinyMCE();
  }

  private GetEmailConfigure(isDirtyUpdate: boolean = false) {
    if (!isDirtyUpdate) this.emailConfigureHeaders = [];

    this.http.get("ICMS", "GetEmailConfigure").subscribe(
      (res) => {
        let confHeadList = res.Table;
        let confGroupList = res.Table1;
        let confDetailList = res.Table2;

        confGroupList = confGroupList.map((confGroup) => {
          confGroup.Vendors = confDetailList
            .filter(
              (confDetail) =>
                confDetail.CONFIGURE_TYPE === confGroup.CONFIGURE_TYPE &&
                confDetail.CONFIGURE_GROUP_ID === confGroup.CONFIGURE_GROUP_ID
            )
            .map((confDetail) => confDetail.VENDOR_CODE);
          return confGroup;
        });

        const confList = confHeadList.map((confHead) => {
          confHead.Groups = confGroupList.filter(
            (confGroup) => confGroup.CONFIGURE_TYPE === confHead.CONFIGURE_TYPE
          );

          return confHead;
        });

        this.emailConfigureHeaders = [...confList];
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  private GetVendorList() {
    this.vendorList = [];
    this.http.get("ICMS", "GetVendorList").subscribe(
      (res) => {
        this.vendorList = res;
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  public InitTinyMCE(isDisabled: boolean = false) {
    this.tinymceOptions = null;
    const _component = this;
    this.tinymceOptions = {
      base_url: "/tinymce",
      suffix: ".min",
      height: 500,
      plugins:
        "importcss image fullscreen table lists link code help wordcount save pagebreak preview",
      body_id: "pdftemplate",
      body_class: "editor",
      importcss_append: true,
      contextmenu_avoid_overlap: ".editor-container",
      contextmenu_never_use_native: true,
      convert_fonts_to_spans: false,
      toolbar_mode: "sliding",
      formats: {
        bold: {
          inline: "span",
          styles: { "font-weight": "bold" },
          exact: true,
        },
      },
      font_family_formats: "Nissan Brand=NissanBrand; Tahoma=Tahoma",
      font_size_formats: [
        "8pt",
        "9pt",
        "10pt",
        "11pt",
        "12pt",
        "14pt",
        "16pt",
        "18pt",
        "20pt",
        "22pt",
        "24pt",
        "30pt",
        "36pt",
        "48pt",
        "60pt",
        "72pt",
        "96pt",
      ],
      toolbar: !isDisabled && [
        // `template`, //Custom button ref
        `save | fontsize forecolor |
        undo redo |
        bold italic underline strikethrough subscript superscript |
        formatselect align numlist bullist outdent indent blockquote | 
        link image table | pagebreak |
        preview print code fullscreen`,
      ],
      menubar: !isDisabled,
      statusbar: !isDisabled,
      pagebreak_separator:
        '<p style="page-break-before: always; clear:both"/></p>',
      pagebreak_split_block: true,
      save_enablewhendirty: false,
      save_onsavecallback: (editor) => {
        // editor.getContent()
        //On Save callback
      },
      setup: function (editor) {
        //Set readonly mode
        if (isDisabled) editor.mode.set("readonly");

        //Custom button
        // editor.ui.registry.addMenuButton('template', {
        //   text: 'Template',
        //   fetch: function (callback) {
        //     const items = [
        //       {
        //         type: 'menuitem',
        //         text: 'Load NMT Brand',
        //         onAction: function () {
        //           editor.insertContent(_component.templateInit.NMT_BRAND);
        //         },
        //       }
        //     ];

        //     callback(items);
        //   },
        // });
      },
    };
  }

  public AddConfigureGroup(confHead: any) {

    this.modal.confirm("Do you want to add new template configure?", this.viewRef, () => {
      confHead.Groups.push({
        CONFIGURE_TYPE: confHead.CONFIGURE_TYPE,
        _id: moment().milliseconds(),
        STATUS: "Y",
      });

    })
  }

  public SaveContent(confGroup: any, editorRef: any) {
    
    this.modal.confirm("Do you want to save",this.viewRef,() => ConfirmSave())
  
    let ConfirmSave = () => {
        const editor = editorRef.editor;
        confGroup.EMAIL_TEMPLATE = editor.getContent();
        this.http.post("ICMS", "SaveEmailTemplate", confGroup).subscribe(
          (res) => {
            if (res === "Done") {
              this.modal.info("Configure saved.", this.viewRef);
              this.GetEmailConfigure(true);
            }
          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
    }
  }


  public OpenSelectVendorDialog(confGroup: any) {
    const unavailableVendors = this.emailConfigureHeaders
      .find((conf) => conf.CONFIGURE_TYPE === confGroup.CONFIGURE_TYPE)
      .Groups.map((group) => {
        if (
          group.CONFIGURE_GROUP_ID !== confGroup.CONFIGURE_GROUP_ID ||
          group._id !== confGroup._id
        ) {
          return group.Vendors;
        } else {
          return [];
        }
      })
      .flat();

    const dialogRef = this.dialog.open(VendorSelectComponent, {
      width: "50vw",
      height: "90vh",
      data: {
        vendorList: this.vendorList,
        selectedList: confGroup.Vendors || [],
        unavailableVendors: unavailableVendors,
      },
    });

    dialogRef.afterClosed().subscribe((selectedVendors) => {
      if (selectedVendors) {
        confGroup.Vendors = selectedVendors || [];
      }
    });
  }

  public ToggleAllVendors(evt: any, confGroup: any, groupLength: number) {
    if (evt.checked && groupLength > 1) {
      this.modal.confirm(
        "PLEASE READ!<br/>After confirm, another configured vendors will be replaced with this configuration.",
        this.viewRef,
        () => {
          confGroup.ALL_VENDOR = evt.checked ? "Y" : "N";
        },
        () => { }
      );
    } else {
      confGroup.ALL_VENDOR = evt.checked ? "Y" : "N";
    }
  }

  public ExpandGroup(groupId: number) {
    this.expandedGroupID = groupId;
  }
}
