import { Component, Input, OnInit, OnChanges, Output, EventEmitter, ContentChildren, TemplateRef, SimpleChanges, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { ActivatedRoute } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { animate, state, style, transition, trigger } from "@angular/animations";
import { MatTableDataSource } from "@angular/material/table";
import { MatPaginator } from "@angular/material/paginator";

/**
 * @param {any} param parameters to use in expand method. 
 * @param {string} method method that call in sPMS (WH Controller) method. 
 * @param {string} tableName Field TABLE_NAME.
 * @param {Array<any>} tableData Detail row list in table. 
 * @event rowClick($event) row click event and callback to parent component.
 * @event rowDBClick($event) row double click event and callback to parent component.
*/

@Component({
  selector: 'snwr040-table',
  templateUrl: './SNWR040_TABLE.component.html',
  styleUrls: ['./SNWR040_TABLE.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class SNWR040_TABLEComponent implements OnChanges {
  constructor(
    private http: HttpService,
    public modal: ModalService,
    public viewref: ViewContainerRef,
    private route: ActivatedRoute,
    private global: Globals
  ) {}

  @Input("isExpand") isExpand: boolean;
  @Input("rowindexExpand") rowindexExpand: Array<number> = [];
  @Input("tableName") tableName: string;
  @Input("tableData") tableData: Array<any> = [];
  @Input("tableHeader") tableHeader: Array<any> = [];
  @Input("tableDetailName") tableDetailName: string;
  @Input("tableDetailData") tableDetailData: any = {};
  @Output("rowClick") rowClick = new EventEmitter<any>();
  @Output("rowDBClick") rowDBClick = new EventEmitter<any>();

  public isCustomContent: boolean = false;
  public isCustomHeader: boolean = false;
  public templateBody: TemplateRef<any>;
  public templateHeader: TemplateRef<any>;
  public clickedRow: any;
  public dataList: any = [];
  public headerList: Array<any> = [];
  public columnsToDisplay = [];

  public expandedElement: null;

  @ContentChildren(TemplateRef) templateRef;
  @ViewChild("paginator", { static: true }) paginator: MatPaginator;

  ngOnInit(): void {
    if (this.tableName) {
      this.GetTableHeader();
    } else if (this.tableHeader.length > 0) {
      this.headerList = this.tableHeader;
      this.ArrangeShowColumn(this.headerList);
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    this.clickedRow = null;
    this.dataList = new MatTableDataSource<any>(this.tableData || []);
    this.dataList.paginator = this.paginator;
    this.ArrangeShowColumn(this.headerList);
    
    if(this.isExpand){
      if (this.tableData.length > 0) {
        console.log("tableDetailData",this.tableDetailData);
        this.expandedElement = this.tableData.find((item) => item.VENDOR_TYPE_NAME == "Total");;
      }
    }
  }

  ngAfterContentInit(): void {
    this.isCustomContent = false;
    this.isCustomHeader = false;
    if (this.templateRef) {
      if (this.templateRef.length == 1) {
        this.isCustomContent = true;
        this.templateBody = this.templateRef._results[0];
      } else if (this.templateRef.length == 2
        && this.templateRef._results.find(t => t._declarationTContainer?.localNames?.includes("tblHeader"))
        && this.templateRef._results.find(t => t._declarationTContainer?.localNames?.includes("tblBody"))) {
        let headerIndex = this.templateRef._results.findIndex(t => t._declarationTContainer?.localNames?.includes("tblHeader"));
        let bodyIndex = this.templateRef._results.findIndex(t => t._declarationTContainer?.localNames?.includes("tblBody"));
        this.isCustomContent = true;
        this.isCustomHeader = true;
        this.templateHeader = this.templateRef._results[headerIndex];
        this.templateBody = this.templateRef._results[bodyIndex];
      }
    }
  }

  private ArrangeShowColumn(data): void {
    this.columnsToDisplay = data.map(v => v.FIELD_NAME);
  }
  
  private GetTableHeader(): void {
    this.http.get("System", "GetHeaderGridView", this.tableName)
    .subscribe(result => {
      this.headerList = result.filter(v => v.VISIBILITY === true);
      this.ArrangeShowColumn(this.headerList);
    })
  }

  public setRowClick(row: any, index: number) {
    if(this.rowClick.observers != null && this.rowClick.observers.length > 0 && this.isExpand && this.rowindexExpand.includes(index)){
      if(this.expandedElement == row){
        this.expandedElement = null;
      }
      else{
        this.expandedElement = row;
      } 
    }
    this.clickedRow = row;
    this.rowClick.emit(row)
  }
  
  public setRowDBClick(row: any, index: number) {
    if(this.rowClick.observers != null && this.rowClick.observers.length > 0 && this.isExpand &&  this.rowindexExpand.includes(index)){
      if(this.expandedElement == row){
        this.expandedElement = null;
      }
      else{
        this.expandedElement = row;
      } 
    }
    this.clickedRow = row;
    this.rowDBClick.emit(row)
  }
}
